﻿package meyermike
{
    import flash.geom.Point;
    import flash.geom.Rectangle;
    import meyermike.collision.CollisionInfo;
    import meyermike.GameObject;
    import meyermike.gamespecific.Constants;
    import meyermike.Layers;
    import meyermike.PlatformerCharacter;
    import meyermike.Time;
    import meyermike.Util;

    /**
     * ...
     * @author Mike Meyer
     */
    public class Physics
    {
        /**
         * Run a physics step of the simulation
         */
        public static function Update():void
        {
            Gravity();
            Collisions();
        }

        /**
         * detect and react to collisions
         */
        protected static function Collisions():void
        {
            Layers.PlayerLayer.ForEach(CallPreCollisionCheck);
            Layers.NPCLayer.ForEach(CallPreCollisionCheck);
            Layers.EnemyLayer.ForEach(CallPreCollisionCheck);
            Layers.ItemLayer.ForEach(CallPreCollisionCheck);

            Layers.ForEachCartesianProductStyle(Layers.PlayerLayer, Layers.PlatformLayer, CharacterPlatformCollision);
            Layers.ForEachCartesianProductStyle(Layers.NPCLayer, Layers.PlatformLayer, CharacterPlatformCollision);
            Layers.ForEachCartesianProductStyle(Layers.EnemyLayer, Layers.PlatformLayer, CharacterPlatformCollision);
            Layers.ForEachCartesianProductStyle(Layers.ItemLayer, Layers.PlatformLayer, CharacterPlatformCollision);

            Layers.PlayerLayer.ForEach(CallPostCollisionCheck);
            Layers.NPCLayer.ForEach(CallPostCollisionCheck);
            Layers.EnemyLayer.ForEach(CallPostCollisionCheck);
            Layers.ItemLayer.ForEach(CallPostCollisionCheck);
        }

        /**
         */
        protected static function CallPreCollisionCheck(character:PlatformerCharacter):void
        {
            character.PreCollisionCheck();
        }
        /**
         */
        protected static function CallPostCollisionCheck(character:PlatformerCharacter):void
        {
            character.PostCollisionCheck();
        }

        /**
         * detect and react to the player colliding with platforms
         */
        protected static function CharacterPlatformCollision(character:PlatformerCharacter, platform:GameObject):void
        {
            if (character.PhysicsEnabled)
            {
                var ci:CollisionInfo = Util.CheckAabbCollision(character, platform, Time.TimeDelta);

                if (null != ci)
                {
                    var converted:CollisionInfo = ci.GlobalToLocal(character.parent);

                    // let the character know he hit something
                    character.CollisionOccurred(converted, platform);
                }
            }
        }


        /**
         * everything that can fall needs to have gravity applied
         */
        protected static function Gravity():void
        {
            Layers.NPCLayer.ForEach(ApplyGravityTo);
            Layers.PlayerLayer.ForEach(ApplyGravityTo);
            Layers.EnemyLayer.ForEach(ApplyGravityTo);
            Layers.BloodLayer.ForEach(ApplyGravityTo);
            Layers.ItemLayer.ForEach(ApplyGravityTo);
        }

        /**
         * apply gravity to the object
         */
        protected static function ApplyGravityTo(go:PlatformerCharacter):void
        {
            if (go.PhysicsEnabled)
            {
                go.dy += Constants.GRAVITY * Time.TimeDelta;

                if (go.dy > PlatformerCharacter.MAX_FALL_SPEED)
                {
                    go.dy = PlatformerCharacter.MAX_FALL_SPEED;
                }
            }
        }
    }

}
