﻿package meyermike.gamespecific
{
    import flash.display.DisplayObject;
    import flash.display.Sprite;
    import meyermike.collision.CollisionInfo;
    import meyermike.InputUtil;
    import meyermike.PlatformerCharacter;

    /**
     * ...
     * @author Mike Meyer
     */
    public class Jumpman extends PlatformerCharacter
    {
        [Embed(source = '../../../assets/boxyguy1.png')]
        private const Frame1Image:Class;
        [Embed(source='../../../assets/boxyguy2.png')]
        private const Frame2Image:Class;

        private var mFrame1:DisplayObject;
        private var mFrame2:DisplayObject;

        private const FRAME_TIME:Number = 0.125;
        private var mAnimTimer:Number;

        private var mWalkDirection:Number;

        public override function Init():void
        {
            super.Init();

            SetPlatformerSettings();
            CreateGraphics();
            Restart();
        }

        /**
         * set the values used to determine the accelerations
         * when controlling the character
         */
        protected function SetPlatformerSettings():void
        {
            mWalkAccel = Constants.PLAYER_WALK_ACCEL;
            mAirAccel = Constants.PLAYER_AIR_ACCEL;
            mStopAccel = Constants.PLAYER_STOP_ACCEL;
            mMaxWalkSpeed = Constants.PLAYER_WALK_MAX_SPEED;
        }

        protected function CreateGraphics():void
        {
            if (null == mFrame1)
            {
                mFrame1 = new Frame1Image();
                mFrame1.x = -0.5 * mFrame1.width;
                mFrame1.y = -0.5 * mFrame1.height;
                this.addChild(mFrame1);
            }

            if (null == mFrame2)
            {
                mFrame2 = new Frame2Image();
                mFrame2.x = -0.5 * mFrame2.width;
                mFrame2.y = -0.5 * mFrame2.height;
                mFrame2.visible = false;
                this.addChild(mFrame2);
            }
        }

        /**
         * per-frame update
         * @param   elapsedTime - time since the last frame
         */
        public override function Update(elapsedTime:Number):void
        {
            super.Update(elapsedTime);

            Walking(elapsedTime);
            Jumping(elapsedTime);
            Animations(elapsedTime);
        }

        /**
         * walking behavior for Jumpman
         */
        public function Walking(elapsedTime:Number):void
        {
            WalkAccel(mWalkDirection, elapsedTime);
        }

        /**
         * jumping behavior for Jumpman
         */
        public function Jumping(elapsedTime:Number):void
        {
            if (IsOnGround)
            {
                var feelerDistFar:Number = Constants.TILE_SIZE + this.height;
                var feelerDistNear:Number = 0.5 * feelerDistFar;
                var offsetToFeet:Number = 0.5 * this.height - 1;
                var inFrontAndBelow:Boolean = (null != Platform.GetTileForPoint(
                                                        this.x + (mWalkDirection * feelerDistNear),
                                                        this.y + offsetToFeet + feelerDistNear ) );
                var inFrontAndAbove:Boolean = (null != Platform.GetTileForPoint(
                                                        this.x + (mWalkDirection * feelerDistNear),
                                                        this.y - feelerDistNear ) );
                var inFrontFar:Boolean = ( null != Platform.GetTileForPoint(
                                                    this.x + (mWalkDirection * feelerDistFar),
                                                    this.y + offsetToFeet ) );
                var inFrontNear:Boolean = ( null != Platform.GetTileForPoint(
                                                    this.x + (mWalkDirection * feelerDistNear),
                                                    this.y + offsetToFeet ) )
                                        ||( null != Platform.GetTileForPoint(
                                                    this.x + (mWalkDirection * feelerDistNear),
                                                    this.y) );
                var above:Boolean = ( null != Platform.GetTileForPoint(
                                                this.x,
                                                this.y - feelerDistNear ) );
                const EPSILON:Number = 4; // how close is "close enough" for the screen bottom check
                var onBottomOfScreen:Boolean = ( this.y + (0.5 * this.height) + EPSILON >= stage.stageHeight);

                if ( !above )
                {
                    if ( (!inFrontAndBelow && !onBottomOfScreen)
                        || inFrontFar
                        || inFrontNear )
                    {
                        this.dy = -Constants.PLAYER_JUMP_SPEED;
                    }
                }
            }
        }

        /**
         * set initial values after spawning
         */
        protected override function Restart():void
        {
            super.Restart();

            mWalkDirection = RIGHT;
            mAnimTimer = 0;
        }


        /**
         * override to deal with collisions
         */
        public override function CollisionOccurred(ci:CollisionInfo, object:Object):void
        {
            super.CollisionOccurred(ci, object);

            var feelerDist:Number = this.width;
            var offsetToFeet:Number = 0.5 * this.height - 4;

            var blockInFront:Boolean = (null != Platform.GetTileForPoint(
                                                    this.x + (mWalkDirection * feelerDist),
                                                    this.y + offsetToFeet) );
            var blockAboveThat:Boolean = (null != Platform.GetTileForPoint(
                                                    this.x + (mWalkDirection * feelerDist),
                                                    this.y - feelerDist) );

            var turnAround:Boolean = (blockInFront && blockAboveThat) || (!blockInFront);

            if (turnAround)
            {
                if (RIGHT == mWalkDirection)
                {
                    if (ci.normal.x < -Math.SQRT1_2)
                    {
                        mWalkDirection = LEFT;
                    }
                }
                else
                {
                    if (ci.normal.x > Math.SQRT1_2)
                    {
                        mWalkDirection = RIGHT;
                    }
                }
            }
        }

        /**
         *
         */
        private function Animations(elapsedTime:Number):void
        {
            mAnimTimer -= elapsedTime;

            if (mAnimTimer <= 0.0)
            {
                mAnimTimer = FRAME_TIME;
                mFrame1.visible = !mFrame1.visible;
                mFrame2.visible = !mFrame2.visible;
            }
        }
    }

}
