﻿package meyermike.gamespecific
{
    import adobe.utils.CustomActions;
    import flash.display.Bitmap;
    import flash.display.DisplayObject;
    import flash.display.DisplayObjectContainer;
    import flash.display.Shape;
    import flash.display.Sprite;
    import flash.display.Stage;
    import flash.events.Event;
    import flash.events.KeyboardEvent;
    import flash.geom.Point;
    import flash.geom.Rectangle;
    import meyermike.collision.CollisionInfo;
    import meyermike.GameInterface;
    import meyermike.GameObject;
    import meyermike.InputUtil;
    import meyermike.Layers;
    import meyermike.gamespecific.scripts.Intro;
    import meyermike.script.Script;
    import meyermike.Time;
    import meyermike.Util;
    import meyermike.widget.TextField;

    /**
     * ...
     * @author Mike Meyer
     */
    public class Main extends Sprite implements GameInterface
    {
        private var mScript:Script;
        private var mNextScript:Script;
        private var mIsPaused:Boolean;
        private static const CAMERA_TIGHTNESS:Number = 0.125;

        private var mCameraFollowTarget:GameObject;

        private var mPauseScreen:Sprite;

        private var mGameHolder:DisplayObjectContainer;

        public function Main():void
        {
            if (stage)
            {
                Init();
            }
            else
            {
                addEventListener(Event.ADDED_TO_STAGE, Init);
            }
        }

        private function Init(e:Event = null):void
        {
            mCameraFollowTarget = null;

            stage.frameRate = 30;
            removeEventListener(Event.ADDED_TO_STAGE, Init);

            mIsPaused = false;

            InputUtil.Init(this);

            addEventListener(Event.ENTER_FRAME, HandleEnterFrame);

            stage.addEventListener(KeyboardEvent.KEY_DOWN, HandleKeyDown);

            SetupScene();

            mNextScript = new Intro();

            stage.addEventListener(Event.DEACTIVATE, Pause);
            stage.addEventListener(Event.ACTIVATE, Unpause);
        }

        /**
         * add the persistent display objects that stick around even between levels
         */
        private function SetupScene():void
        {
            // holder for the game itself (gets moved to scroll the camera)
            mGameHolder = new Sprite();
            this.addChild(mGameHolder);

            // call it once so that the layers know what the container is
            Layers.CreateLayers(mGameHolder);

            CreatePauseIndicator();
        }

        /**
         * create the pause screen
         */
        protected function CreatePauseIndicator():void
        {
            mPauseScreen = new Sprite();
            mPauseScreen.graphics.beginFill( Constants.PAUSE_SCREEN_COLOR );
            mPauseScreen.graphics.drawRect( 0, 0, stage.stageWidth, stage.stageHeight )
            mPauseScreen.graphics.endFill();
            mPauseScreen.alpha = Constants.PAUSE_SCREEN_ALPHA;
            mPauseScreen.visible = false;

            var pauseLabel:TextField = new TextField();
            pauseLabel.SetText("PAUSED");
            pauseLabel.SetFont(Constants.GAME_FONT);
            pauseLabel.SetFontSize(36);
            pauseLabel.SetFontColor(Constants.FONT_COLOR);
            pauseLabel.CenterOn( 0.5 * stage.stageWidth, 0.4 * stage.stageHeight );
            mPauseScreen.addChild(pauseLabel);

            var clickInstructions:TextField = new TextField();
            clickInstructions.SetText("click to continue");
            clickInstructions.SetFont(Constants.GAME_FONT);
            clickInstructions.SetFontSize(16);
            clickInstructions.SetFontColor(Constants.FONT_COLOR);
            clickInstructions.CenterOn( 0.5 * stage.stageWidth, 0.7 * stage.stageHeight );
            mPauseScreen.addChild(clickInstructions);

            this.addChild(mPauseScreen);
        }

        private function HandleEnterFrame(e:Event = null):void
        {
            Time.TimeDelta = (1.0 / stage.frameRate);

            ScriptChange();

            if (!mIsPaused)
            {
                Update();
                Integrate();
                PreDraw();
            }
        }

        private function Update():void
        {
            mScript.Update();
            Layers.Update();
            InputUtil.Update();
        }

        private function Integrate():void
        {
            Layers.Integrate();
        }
        private function PreDraw():void
        {
            Layers.PreDraw();
        }

        /// Called on each Beat of the game.
        private function Beat(e:Event = null):void
        {
            var beat:Number = mScript.GetCurBeat();

            Layers.Beat(beat);
        }

        private function OffBeat(e:Event = null):void
        {
            var beat:Number = mScript.GetCurBeat();

            Layers.OffBeat(beat);
        }
        private function QuarterBeat(e:Event = null):void
        {
            var beat:Number = mScript.GetCurQuarterBeat();

            Layers.QuarterBeat(beat);
        }

        public function Pause(e:Event = null):void
        {
            mIsPaused = true;
            mPauseScreen.visible = true;
            Music.inst.Pause();
        }
        public function Unpause(e:Event = null):void
        {
            mIsPaused = false;
            mPauseScreen.visible = false;
            Music.inst.Resume();
        }

        /**
         * if a script change was queued up, perform the actual switch now
         */
        private function ScriptChange():void
        {
            if (mNextScript != null)
            {
                EndScript(mScript);
                BeginScript(mNextScript);

                mScript = mNextScript;
                mNextScript = null;
            }
        }

        /**
         * shut it down
         */
        private function EndScript(script:Script):void
        {
            if (script != null)
            {
                script.removeEventListener(Script.EVENT_BEAT, Beat);
                script.removeEventListener(Script.EVENT_OFFBEAT, OffBeat);
                script.removeEventListener(Script.EVENT_QUARTERBEAT, QuarterBeat);
                script.End();
            }
        }

        /**
         *
         */
        private function BeginScript(script:Script):void
        {
            if (script != null)
            {
                script.addEventListener(Script.EVENT_BEAT, Beat);
                script.addEventListener(Script.EVENT_OFFBEAT, OffBeat);
                script.addEventListener(Script.EVENT_QUARTERBEAT, QuarterBeat);
                script.Begin(this);
            }
        }

        /**
         * access to the script
         */
        public function GetScript():Script
        {
            return mScript;
        }

        /**
         * set a script to switch to once the current beat/update is finished
         */
        public function SwitchToScript(nextScript:Script):void
        {
            mNextScript = nextScript;
        }

        /**
         * Cheat codes!
         */
        private var mCheatString:String = "";
        private static const KEYLOG_MAX_LENGTH:uint = 16;
        private function HandleKeyDown(e:KeyboardEvent):void
        {
            if (0 != e.charCode)
            {
                mCheatString += String.fromCharCode(e.charCode);

                if (mCheatString.length > KEYLOG_MAX_LENGTH)
                {
                    mCheatString = mCheatString.substring(1);
                }

                // next track cheat
                if (0 == mCheatString.indexOf("next"))
                {
                    //Music.StartNewMusic();
                    mCheatString = "";
                }

                // restart game cheat
                if (-1 != mCheatString.indexOf("restart"))
                {
                    // do something here
                    ResetScrolling();
                    SwitchToScript(new Intro());
                    mCheatString = "";
                }
            }
        }

        /**
         * go back to the starting position
         */
        public function ResetScrolling():void
        {
            mGameHolder.x = 0;
            mGameHolder.y = 0;
        }

        /**
         * give it a target to scroll towards
         */
        public function  SetCameraFollowTarget(go:GameObject):void
        {
            mCameraFollowTarget = go;
        }

        /**
         * Allow access to the "camera" position, so that objects can follow the camera.
         * NOTE this actually tells you where the upper-left corner is in the scene
         */
        public function GetCameraPosition():Point
        {
            return new Point( -mGameHolder.x, -mGameHolder.y);
        }

        /**
         * set how much the game is zoomed in
         */
        public function SetZoom(zoomLevel:Number):void
        {
            mGameHolder.scaleX = zoomLevel;
            mGameHolder.scaleY = zoomLevel;
        }

        /**
         * make the stage available to subsystems
         */
        public function GetStage():Stage
        {
            return this.stage;
        }

        /**
         * a game should have a or be a DisplayObjectContainer so that scripts
         * can attach things
         */
        public function GetDisplayObjectContainer():DisplayObjectContainer
        {
            return this;
        }
    }
}
