﻿package meyermike.gamespecific
{
    import flash.display.Bitmap;
    import flash.display.DisplayObject;
    import flash.display.Sprite;
    import flash.geom.Point;
    import flash.geom.Rectangle;
    import meyermike.GameObject;
    import meyermike.Layers;
    import meyermike.Time;

    /**
     * ...
     * @author Mike Meyer
     */
    public class Platform extends GameObject
    {
        [Embed(source = '../../../assets/Brick.png')]
        private const BrickImg:Class;

        private var mGraphic:DisplayObject;

        private var _scrollable:Boolean = true;
        public function get Scrollable():Boolean { return _scrollable; }
        public function set Scrollable(value:Boolean):void { _scrollable = value; }

        private static var sTileScrollAmount:Number = 0;

        /// workaround for horrible Flash rounding of .x,.y
        public var ScrollError:Number = 0;

        /**
         * If there is a platform already there for the specified point, return
         * it.  Otherwise, return null.
         */
        public static function GetTileForPoint(x:Number, y:Number):Platform
        {
            for each ( var platform:Platform in Layers.PlatformLayer.GetObjects() )
            {
                if ( platform.getBounds(Layers.PlatformLayer.GetContainer()).containsPoint(new Point(x, y)) )
                {
                    return platform;
                }
            }

            // failed to find it in the above loop
            return null;
        }
        /**
         * Get the tile location for the tile that would contain the point
         */
        public static function GetTileLocationNearest(x:Number, y:Number):Point
        {
            return new Point(
                Constants.TILE_SIZE * Math.floor( x / Constants.TILE_SIZE ),
                (Constants.TILE_SIZE * Math.floor( (y - sTileScrollAmount) / Constants.TILE_SIZE )) + sTileScrollAmount );
        }
        /**
         * scroll the tiles down
         */
        public static function ScrollTiles():void
        {
            var amount:Number = Constants.TILE_SCROLL_RATE * Time.TimeDelta;

            for each ( var platform:Platform in Layers.PlatformLayer.GetObjects() )
            {
                if (platform.Scrollable)
                {
                    var desiredY:Number = platform.y + amount + platform.ScrollError;

                    platform.y = desiredY;

                    platform.ScrollError = desiredY - platform.y;
                }
            }
            sTileScrollAmount += amount;

            // pop back a row once we pass TILE_SIZE
            if (sTileScrollAmount >= Constants.TILE_SIZE)
            {
                sTileScrollAmount -= Constants.TILE_SIZE;
            }
        }

        public override function Init():void
        {
            super.Init();

            CreateGraphics();

            ScrollError = 0;
        }

        protected function CreateGraphics():void
        {
            if (null == mGraphic)
            {
                var gfx:Bitmap = new BrickImg();
                this.addChild(gfx);
                mGraphic = gfx;
            }
        }

        /**
         * by default a GO never dies, but we need offscreen platforms to go away
         */
        public override function IsAlive():Boolean
        {
            var box:Rectangle = this.GetCollisionBounds();

            if (box.right < 0)
                return false;
            if (box.left > stage.stageWidth)
                return false;
            if (box.bottom < 0)
                return false;
            if (box.top > stage.stageHeight)
                return false;

            return true;
        }
    }

}
