﻿package meyermike.gamespecific
{
    import flash.display.DisplayObject;
    import flash.display.Sprite;
    import flash.geom.Point;
    import meyermike.collision.CollisionInfo;
    import meyermike.GameObject;
    import meyermike.InputUtil;
    import meyermike.Layers;
    import meyermike.PlatformerCharacter;

    /**
     * ...
     * @author Mike Meyer
     */
    public class Player extends PlatformerCharacter
    {
        [Embed(source='../../../assets/player_walk1.png')]
        private const Frame1Image:Class;
        [Embed(source='../../../assets/player_walk2.png')]
        private const Frame2Image:Class;

        private var mFrame1:DisplayObject;
        private var mFrame2:DisplayObject;

        private const FRAME_TIME:Number = 0.125;
        private var mAnimTimer:Number;

        private var mAirJumpsRemaining:Number = Constants.NUM_AIR_JUMPS;

        // key they used to jump, so we know which key's release event we care about
        private var mJumpKey:uint = 0;

        private var mIsJumping:Boolean = false;
        public function get IsJumping():Boolean { return mIsJumping; }
        public function set IsJumping(value:Boolean):void { mIsJumping = value; }

        private const MODE_PLATFORMER:Number        = 0;
        private const MODE_BLOCK_PLACEMENT:Number   = 1;
        private var mMode:Number;

        public override function Init():void
        {
            super.Init();

            SetPlatformerSettings();
            CreateGraphics();
            Restart();
        }

        /**
         * set the values used to determine the accelerations
         * when controlling the character
         */
        protected function SetPlatformerSettings():void
        {
            mWalkAccel = Constants.PLAYER_WALK_ACCEL;
            mAirAccel = Constants.PLAYER_AIR_ACCEL;
            mStopAccel = Constants.PLAYER_STOP_ACCEL;
            mMaxWalkSpeed = Constants.PLAYER_WALK_MAX_SPEED;
        }

        protected function CreateGraphics():void
        {
            if (null == mFrame1)
            {
                mFrame1 = new Frame1Image();
                mFrame1.x = -0.5 * mFrame1.width;
                mFrame1.y = -0.5 * mFrame1.height;
                this.addChild(mFrame1);
            }

            if (null == mFrame2)
            {
                mFrame2 = new Frame2Image();
                mFrame2.x = -0.5 * mFrame2.width;
                mFrame2.y = -0.5 * mFrame2.height;
                mFrame2.visible = false;
                this.addChild(mFrame2);
            }
        }

        /**
         * per-frame update
         * @param   elapsedTime - time since the last frame
         */
        public override function Update(elapsedTime:Number):void
        {
            super.Update(elapsedTime);

            if (MODE_PLATFORMER == mMode)
            {
                Walk(elapsedTime);
                Jumping(elapsedTime);
                ActionButton(elapsedTime);
            }
            else
            {
                TilePlacement(elapsedTime);
            }

            Animations(elapsedTime);
        }

        /**
         * change dx based on user input
         */
        protected function Walk(elapsedTime:Number):void
        {
            var desiredDirection:Number = STOP;
            if (InputUtil.IsDown(Constants.MOVE_RIGHT_KEY))
            {
                desiredDirection = RIGHT;
            }
            else if (InputUtil.IsDown(Constants.MOVE_LEFT_KEY))
            {
                desiredDirection = LEFT;
            }

            WalkAccel(desiredDirection, elapsedTime);
        }

        /**
         *
         */
        protected function Jumping(elapsedTime:Number):void
        {
            if ( InputUtil.IsUp(mJumpKey) || IsOnGround)
            {
                this.IsJumping = false;
                mJumpKey = 0;
            }

            // if we aren't holding the jump button anymore, cut
            // off the jump height
            if (!IsOnGround && !this.IsJumping && (this.dy < 0) )
            {
                this.dy += Constants.PLAYER_JUMP_STOP_ACCEL * elapsedTime;

                // don't shoot us down, though
                if (this.dy > 0)
                {
                    this.dy = 0;
                }
            }

            // starting a new jump
            var justPressedJumpKey:uint = 0;
            for each (var keyCode:uint in Constants.JUMP_KEYS)
            {
                if (InputUtil.WasJustPressed(keyCode))
                {
                    justPressedJumpKey = keyCode;
                }
            }

            if (InputUtil.WasJustPressed(justPressedJumpKey))
            {
                if (IsOnGround || (mAirJumpsRemaining > 0) )
                {
                    this.dy = -Constants.PLAYER_JUMP_SPEED;
                    this.IsJumping = true;
                    mJumpKey = justPressedJumpKey;
                }
                if (!IsOnGround)
                {
                    --mAirJumpsRemaining;
                }
            }

            // reset air jumps after landing
            if (IsOnGround)
            {
                mAirJumpsRemaining = Constants.NUM_AIR_JUMPS;
            }
        }

        /**
         *
         */
        protected function ActionButton(elapsedTime:Number):void
        {
            for each (var keyCode:uint in Constants.ACTION_KEYS)
            {
                if (InputUtil.WasJustPressed(keyCode))
                {
                    if (null == Platform.GetTileForPoint(this.x, this.y))
                    {
                        var loc:Point = Platform.GetTileLocationNearest(this.x, this.y + (0.25 * this.height) );
                        this.x = loc.x + ( 0.5 * Constants.TILE_SIZE );
                        this.y = loc.y + ( 0.5 * Constants.TILE_SIZE );
                        this.dx = 0;
                        this.dy = 0;
                        mMode = MODE_BLOCK_PLACEMENT;
                        this.PhysicsEnabled = false;

                        var platform:GameObject = Layers.PlatformLayer.Add(Platform);
                        platform.x = loc.x;
                        platform.y = loc.y;
                    }
                }
            }
        }

        /**
         *
         */
        protected function TilePlacement(elapsedTime:Number):void
        {
            var blockPlaced:Boolean = false;

            if (InputUtil.WasJustPressed(Constants.MOVE_RIGHT_KEY))
            {
                this.x += Constants.TILE_SIZE;
                blockPlaced = true;
            }
            else if (InputUtil.WasJustPressed(Constants.MOVE_LEFT_KEY))
            {
                this.x -= Constants.TILE_SIZE;
                blockPlaced = true;
            }
            else if (InputUtil.WasJustPressed(Constants.MOVE_UP_KEY))
            {
                this.y -= Constants.TILE_SIZE;
                blockPlaced = true;
            }
            else if (InputUtil.WasJustPressed(Constants.MOVE_DOWN_KEY))
            {
                this.y += Constants.TILE_SIZE;
                blockPlaced = true;
            }

            if (blockPlaced)
            {
                this.PhysicsEnabled = true;
                mMode = MODE_PLATFORMER;
            }
        }

        /**
         * set initial values after spawning
         */
        protected override function Restart():void
        {
            super.Restart();
            mMode = MODE_PLATFORMER;
            mIsJumping = false;
            mAnimTimer = 0;
        }

        /**
         *
         */
        private function Animations(elapsedTime:Number):void
        {
            mAnimTimer -= elapsedTime;

            if (mAnimTimer <= 0.0)
            {
                mAnimTimer = FRAME_TIME;
                mFrame1.visible = !mFrame1.visible;
                mFrame2.visible = !mFrame2.visible;
            }
        }
    }

}
