﻿package meyermike.gamespecific.scripts
{
    import flash.events.MouseEvent;
    import flash.display.Stage;
    import flash.events.Event;
    import flash.geom.Rectangle;
    import meyermike.collision.CollisionInfo;
    import meyermike.GameInterface;
    import meyermike.GameObject;
    import meyermike.gamespecific.Constants;
    import meyermike.gamespecific.Item;
    import meyermike.gamespecific.Jumpman;
    import meyermike.gamespecific.Music;
    import meyermike.gamespecific.Platform;
    import meyermike.gamespecific.Player;
    import meyermike.Layers;
    import meyermike.music.MusicEvent;
    import meyermike.Physics;
    import meyermike.PlatformerCharacter;
    import meyermike.script.Script;
    import meyermike.script.Step;
    import meyermike.script.step.Loop;
    import meyermike.script.step.RunFunc;
    import meyermike.script.step.Wait;
    import meyermike.Time;
    import meyermike.Util;
    import meyermike.widget.PulsingTextButton;
    import meyermike.widget.TextField;

    /**
     * ...
     * @author Mike Meyer
     */
    public class Game extends Script
    {
        [Embed(source='../../../../assets/sky.png')]
        private const BackgroundImage:Class;

        private var mPlayer:Player;
        private var mJumpman:Jumpman;

        private var mMusicLabel:TextField;
        private var mMusicTitle:TextField;
        private var mMusicArtist:TextField;
        private var mMusicLink:TextField;

        private var mControls:TextField;

        private const TEXT_FADE_TIME:Number = 5.0;
        private const TEXT_FADE_DELAY:Number = 3.0;

        private var mResetButton:PulsingTextButton;

        private var mFadeMusic:Boolean;
        private var mMusicFadeTimer:Number;

        private var mFadeControls:Boolean;
        private var mControlsFadeTimer:Number;

        private var mMusicFadeDelayTimer:Number;

        /**
         * initial setup
         */
        public override function Begin(game:GameInterface):void
        {
            super.Begin(game);

            mFadeMusic = false;
            mMusicFadeTimer = 0.0;
            mFadeControls = false;
            mControlsFadeTimer = 0.0;
            mMusicFadeDelayTimer = 0.0;


            // working around flash issue with clicked textfield making the keyboard not work.
            mGame.GetDisplayObjectContainer().stage.focus = null;

            var stage:Stage = game.GetStage();

            mPlayer = Layers.PlayerLayer.Add(Player) as Player;
            mPlayer.x = 0.25 * stage.stageWidth;
            mPlayer.y = 0.25 * stage.stageHeight;

            mJumpman = Layers.NPCLayer.Add(Jumpman) as Jumpman;
            mJumpman.x = 0.75 * stage.stageWidth;
            mJumpman.y = 0.25 * stage.stageHeight;

            AddTextFields();
            Music.inst.addEventListener(MusicEvent.EVENT_NEW_SONG, SongChanged);
            Music.inst.StartNewMusic();

            var bg:GameObject = Layers.BackgroundLayer.Add(GameObject);
            bg.addChild(new BackgroundImage());

            AddResetButton();
        }

        /**
         *
         */
        public function AddTextFields():void
        {
            mMusicLabel  = Util.AddTextfield("Music", 0.01, 0.005, 16);
            mMusicTitle  = Util.AddTextfield("", 0.03, 0.03, 14);
            mMusicArtist = Util.AddTextfield("", 0.03, 0.05, 14);
            mMusicLink   = Util.AddTextfield("", 0.03, 0.07, 14);

            mControls = Util.AddTextfield("", 0.03, 0.9, 18);
        }

        /**
         *
         */
        private function SongChanged(e:MusicEvent):void
        {
            mMusicTitle.SetText( e.song.title );
            mMusicArtist.SetText( e.song.artist );
            mMusicLink.SetText( e.song.link );

            mMusicFadeDelayTimer = 0;

            mMusicLabel.alpha = 1;
            mMusicTitle.alpha = 1;
            mMusicArtist.alpha = 1;
            mMusicLink.alpha = 1;
        }

        /**
         * Update and check timer
         */
        public override function Update():void
        {
            super.Update();

            //Layers.ForEachCartesianProductStyle(Layers.PlayerLayer, Layers.EnemyLayer, PlayerEnemyCheck)

            Platform.ScrollTiles();

            Physics.Update();

            FreeTrappedCharacters();
            ScreenEdgeCollisions();

            // check for coin collecting
            Layers.ForEachCartesianProductStyle(Layers.NPCLayer, Layers.ItemLayer, CoinCollectCheck);

            // text fading
            HandleTextFading();
        }

        /**
         * clean up
         */
        public override function End():void
        {
            super.End();

            Music.inst.removeEventListener(MusicEvent.EVENT_NEW_SONG, SongChanged);
            mResetButton.removeEventListener(MouseEvent.CLICK, Reset);

            mPlayer = null;
            mJumpman = null;

            mGame.SetCameraFollowTarget(null);
            mGame.ResetScrolling();
            Layers.RemoveAll();
        }

        /**
         * Keep the player and NPC on screen at all times
         */
        protected function ScreenEdgeCollisions():void
        {
            KeepOnScreen(mPlayer);
            KeepOnScreen(mJumpman);

            for each ( var item:Item in Layers.ItemLayer.GetObjects() )
            {
                KeepOnScreen(item)
            }
        }


        /**
         * Keep the player and NPC on screen at all times
         */
        protected function KeepOnScreen(character:PlatformerCharacter):void
        {
            if (character.PhysicsEnabled)
            {

                var collisionBounds:Rectangle = character.GetCollisionBounds();
                var ci:CollisionInfo = null;
                var converted:CollisionInfo = null;

                if (collisionBounds.left < 0)
                {
                    ci = new CollisionInfo();
                    ci.location.x = 0;
                    ci.location.y = character.y;
                    ci.normal.x = 1;
                    ci.normal.y = 0;
                    ci.t = 0;

                    converted = ci.GlobalToLocal(character.parent);

                    // let the character know he hit something
                    character.CollisionOccurred(converted, null);
                }
                if (collisionBounds.right > character.stage.stageWidth)
                {
                    ci = new CollisionInfo();
                    ci.location.x = character.stage.stageWidth;
                    ci.location.y = character.y;
                    ci.normal.x = -1;
                    ci.normal.y = 0;
                    ci.t = 0;

                    converted = ci.GlobalToLocal(character.parent);
                    // let the character know he hit something
                    character.CollisionOccurred(converted, null);
                }
                if (collisionBounds.top < 0)
                {
                    ci = new CollisionInfo();
                    ci.location.x = character.x;
                    ci.location.y = 0;
                    ci.normal.x = 0;
                    ci.normal.y = 1;
                    ci.t = 0;

                    converted = ci.GlobalToLocal(character.parent);
                    // let the character know he hit something
                    character.CollisionOccurred(converted, null);
                }
                if (collisionBounds.bottom > character.stage.stageHeight)
                {
                    ci = new CollisionInfo();
                    ci.location.x = character.x;
                    ci.location.y = character.stage.stageHeight;
                    ci.normal.x = 0;
                    ci.normal.y = -1;
                    ci.t = 0;

                    converted = ci.GlobalToLocal(character.parent);
                    // let the character know he hit something
                    character.CollisionOccurred(converted, null);
                }
            }
        }

        /**
         * if he's getting caught between things, move him up
         */
        protected function FreeTrappedCharacters():void
        {
            FreeIfTrapped(mPlayer);
            FreeIfTrapped(mJumpman);
        }

        /**
         * if he's getting caught between things, move him up
         */
        protected function FreeIfTrapped(character:PlatformerCharacter):void
        {
            if (character.PhysicsEnabled)
            {
                while ( null != Platform.GetTileForPoint( character.x, character.y ) )
                {
                    character.y -= Constants.TILE_SIZE;
                }
            }
        }

        /**
         * steps to execute in this script
         */
        protected override function CreateScript():Step
        {
            var fadeControls:Step = new Loop(1, [
                new Wait(TEXT_FADE_DELAY-1),
                new RunFunc(FadeControls),
                new Wait(TEXT_FADE_TIME-1)
                ] );

            var steps:/*Step*/Array = [
                new Wait(1),

                new RunFunc(ShowControls_Part1),
                fadeControls,

                new RunFunc(ShowControls_Part2),
                fadeControls,

                new RunFunc(ShowControls_Part3),
                fadeControls,

                new Loop( -1, [
                    new RunFunc(SpawnItem),
                    new Wait(Constants.SECONDS_BETWEEN_ITEMS)
                    ] )
                ];
            return new Loop(1, steps);
        }

        /**
         * Add an Item to the scene
         */
        protected function SpawnItem():void
        {
            if (Layers.ItemLayer.GetObjects().length < Constants.MAX_COINS)
            {
                var item:Item = Layers.ItemLayer.Add(Item) as Item;

                const SPAWN_MARGIN:Number = 20;
                const MAXTRIES:Number = 20;
                var tryCount:Number = 0;

                do
                {
                    item.x = Util.Lerp(SPAWN_MARGIN, item.stage.stageWidth-SPAWN_MARGIN, Math.random());
                    item.y = Util.Lerp(item.stage.stageHeight*0.25, item.stage.stageHeight-SPAWN_MARGIN, Math.random());

                    ++tryCount;
                }
                while ( (tryCount < MAXTRIES) && (null != Platform.GetTileForPoint(item.x, item.y) ) );
            }
        }

        /**
         *
         */
        protected function CoinCollectCheck(character:PlatformerCharacter, item:Item):void
        {
            if (!item.Dying && !character.Dying)
            {
                if (null != Util.CheckAabbCollision(character, item, Time.TimeDelta) )
                {
                    item.Collected();
                }
            }
        }

        /**
         *
         */
        private function AddResetButton():void
        {
            mResetButton = Layers.UILayer.Add(PulsingTextButton) as PulsingTextButton;
            mResetButton.SetText("reset");
            mResetButton.SetFont(Constants.GAME_FONT);
            mResetButton.SetFontSize(18);
            mResetButton.SetFontColor(Constants.FONT_COLOR);
            mResetButton.SetHoverColor(Constants.HIGHLIGHT_COLOR);
            mResetButton.CenterOn(
                0.9 * mGame.GetStage().stageWidth,
                0.03 * mGame.GetStage().stageHeight);
            mResetButton.addEventListener(MouseEvent.CLICK, Reset);
        }

        /**
         * reset if you got stuck and don't feel like waiting
         */
        private function Reset(e:Event = null):void
        {
            Restart();
        }

        /**
         * add the textfields to show the controls
         */
        private function ShowControls_Part1():void
        {
            mControls.SetText("Left and Right arrow keys to move");
            mControls.alpha = 1;
        }

        /**
         * add the textfields to show the controls
         */
        private function ShowControls_Part2():void
        {
            mControls.SetText("'Z' to jump");
            mControls.alpha = 1;
        }

        /**
         * add the textfields to show the controls
         */
        private function ShowControls_Part3():void
        {
            mControls.SetText("'X' to place a block");
            mControls.alpha = 1;
        }

        /**
         *
         */
        private function FadeControls():void
        {
            mFadeControls = true;
            mControlsFadeTimer = 0;
        }

        /**
         *
         */
        private function FadeMusic():void
        {
            mFadeMusic = true;
            mMusicFadeTimer = 0;
        }

        /**
         *
         */
        private function HandleTextFading():void
        {
            if (mMusicFadeDelayTimer < TEXT_FADE_DELAY)
            {
                mMusicFadeDelayTimer += Time.TimeDelta;

                if (mMusicFadeDelayTimer >= TEXT_FADE_DELAY)
                {
                    FadeMusic();
                }
            }

            var alpha:Number;

            if (mFadeMusic)
            {
                mMusicFadeTimer += Time.TimeDelta;
                alpha = 1.0 - (mMusicFadeTimer / TEXT_FADE_TIME);
                mMusicLabel.alpha = alpha;
                mMusicTitle.alpha = alpha;
                mMusicArtist.alpha = alpha;
                mMusicLink.alpha = alpha;

                if (mMusicFadeTimer >= TEXT_FADE_TIME)
                {
                    mFadeMusic = false;
                }
            }

            if (mFadeControls)
            {
                mControlsFadeTimer += Time.TimeDelta;
                alpha = 1.0 - (mControlsFadeTimer / TEXT_FADE_TIME);
                mControls.alpha = alpha;

                if (mControlsFadeTimer >= TEXT_FADE_TIME)
                {
                    mFadeControls = false;
                }
            }
        }
    }
}
