﻿package meyermike.minild10
{
    import flash.display.Bitmap;
    import flash.geom.ColorTransform;
    import flash.geom.Rectangle;
    import meyermike.PlatformerCharacter;

    /**
     * ...
     * @author Mike Meyer
     */
    public class MiniLD10Character extends PlatformerCharacter
    {
        [Embed(source='../../../assets/character_idle.png')]
        private static var IdleGraphic:Class;

        [Embed(source='../../../assets/character_walk3.png')]
        private static var JumpGraphic:Class;

        [Embed(source='../../../assets/character_punch.png')]
        private static var PunchGraphic:Class;

        private var mIdle:Bitmap;
        private var mRun:CharacterRunAnimation;
        private var mJump:Bitmap;
        private var mPunch:Bitmap;

        private var mPunchTimer:Number;

        public function get IsPunching():Boolean { return mPunchTimer > 0.0; }

        protected function CreateGraphics(color:uint):void
        {
            if (null == mIdle)
            {
                mIdle = new IdleGraphic();
                mJump = new JumpGraphic();
                mPunch = new PunchGraphic();

                var newBitmaps:Array = [mIdle, mJump, mPunch];
                for each (var bmp:Bitmap in newBitmaps)
                {
                    bmp.smoothing = false;
                    bmp.scaleX = 2.0;
                    bmp.scaleY = 2.0;
                    bmp.x = -0.5 * bmp.width;
                    bmp.y = -0.5 * bmp.height;
                    this.addChild(bmp);
                }

                mRun = new CharacterRunAnimation();
                this.addChild(mRun);

                var colorTransform:ColorTransform = this.transform.colorTransform;
                colorTransform.redMultiplier   = ( (color & 0xFF0000) >> 16) / 255;
                colorTransform.greenMultiplier = ( (color & 0x00FF00) >> 8) / 255;
                colorTransform.blueMultiplier  = ( (color & 0x0000FF) ) / 255;
                this.transform.colorTransform  = colorTransform;
            }
        }

        /**
         * determine which animation should be playing
         */
        protected function Animation():void
        {
            // switch to the correct animation
            if (IsPunching)
            {
                SwitchToPunchAnim();
            }
            else
            {
                if (IsOnGround)
                {
                    if (this.dx == 0.0)
                    {
                        SwitchToIdleAnim();
                    }
                    else
                    {
                        SwitchToRunAnim();
                    }
                }
                else
                {
                    SwitchToJumpAnim();
                }
            }
        }

        /**
         * Get the collision bounds for collision testing
         */
        public override function GetCollisionBounds():Rectangle
        {
            // bigger box when punching
            if (!IsPunching)
                this.scaleX *= 0.5;

            var bounds:Rectangle = this.getBounds(this.root);

            if (!IsPunching)
                this.scaleX *= 2.0;

            return bounds;
        }


        /**
         *
         */
        private function SwitchToIdleAnim():void
        {
            mIdle.visible = true;
            mRun.visible = false;
            mJump.visible = false;
            mPunch.visible = false;
        }

        /**
         *
         */
        private function SwitchToRunAnim():void
        {
            mIdle.visible = false;
            mRun.visible = true;
            mJump.visible = false;
            mPunch.visible = false;
        }

        /**
         *
         */
        private function SwitchToJumpAnim():void
        {
            mIdle.visible = false;
            mRun.visible = false;
            mJump.visible = true;
            mPunch.visible = false;
        }

        /**
         *
         */
        private function SwitchToPunchAnim():void
        {
            mIdle.visible = false;
            mRun.visible = false;
            mJump.visible = false;
            mPunch.visible = true;
        }


        /**
         * per-frame update
         * @param   elapsedTime - time since the last frame
         */
        public override function Update(elapsedTime:Number):void
        {
            super.Update(elapsedTime);

            mRun.Update(elapsedTime);

            Walk(elapsedTime);
            Jumping(elapsedTime);
            Punching(elapsedTime);

            Animation();
        }

        /**
         * change dx based on user input
         */
        protected function Walk(elapsedTime:Number):void
        {
        }

        /**
         *
         */
        protected function Jumping(elapsedTime:Number):void
        {
        }

        /**
         * determine which animation should be playing
         */
        protected function Punching(elapsedTime:Number):void
        {
            if (0.0 < mPunchTimer)
            {
                mPunchTimer -= elapsedTime;
            }
        }

        /**
         *
         */
        protected function StartPunch():void
        {
            if (!IsPunching)
                mPunchTimer = Constants.PUNCH_DURATION;
        }

        /**
         *
         */
        protected override function Restart():void
        {
            super.Restart();
            mPunchTimer = 0.0;
        }

        /**
         * get rid of these when they fall down too far
         */
        public override function IsAlive():Boolean
        {
            return (this.getBounds(stage).top < stage.stageHeight);
        }
    }

}
