﻿package meyermike.minild10.scripts
{
    import flash.display.DisplayObject;
    import flash.display.Sprite;
    import flash.events.Event;
    import flash.geom.Rectangle;
    import meyermike.collision.CollisionInfo;
    import meyermike.constants.Colors;
    import meyermike.GameInterface;
    import meyermike.GameObject;
    import meyermike.Layers;
    import meyermike.minild10.Constants;
    import meyermike.minild10.Enemy;
    import meyermike.minild10.Family;
    import meyermike.minild10.MiniLD10Character;
    import meyermike.minild10.Physics;
    import meyermike.minild10.Player;
    import meyermike.minild9.Blood;
    import meyermike.PlatformerCharacter;
    import meyermike.script.Script;
    import meyermike.script.Step;
    import meyermike.script.step.CreateGameObjectFromBitmap;
    import meyermike.script.step.Loop;
    import meyermike.script.step.SpawnGameObject;
    import meyermike.script.step.Wait;
    import meyermike.Util;
    import meyermike.Time;
    import meyermike.widget.TextField;

    /**
     * ...
     * @author Mike Meyer
     */
    public class Game extends Script
    {
        [Embed(source = '../../../../assets/Fonts/DisposableDroidBB.ttf', fontFamily='TimerFont')]
        private static var Font:Class;

        private var mEnemiesPunched:uint = 0;

        private var mTextFields:/*TextField*/Array;

        public function Game()
        {
            this.mBeatsPerMinute = 240.0;
            super();
        }

        /**
         * initial setup
         */
        public override function Begin(game:GameInterface):void
        {
            super.Begin(game);

            // working around flash issue with clicked textfield making the keyboard not work.
            mGame.GetDisplayObjectContainer().stage.focus = null;

            mEnemiesPunched = 0;

            // player
            var player:Player = Layers.PlayerLayer.Add(Player) as Player;
            player.addEventListener(PlatformerCharacter.EVENT_DEATH, CharacterDied);
            mGame.SetCameraFollowTarget(player);
            player.x = 100;
            player.y = 215;

            AddPlatforms();

            mTextFields = new Array();

            var textField:meyermike.widget.TextField = new meyermike.widget.TextField();
            mGame.GetDisplayObjectContainer().addChild(textField);
            textField.SetText("");
            textField.SetFont("TimerFont");
            textField.SetFontSize(24);
            textField.SetFontColor(Colors.WHITE_ENOUGH);
            textField.CenterOn(0.5 * Constants.STAGE_WIDTH, 0.05 * Constants.STAGE_HEIGHT);
            mTextFields.push(textField);
        }

        /**
         * Update the label
         */
        private function UpdateScore():void
        {
            mTextFields[0].SetText("Enemies Punched:  "+mEnemiesPunched);
            mTextFields[0].CenterOn(0.5 * Constants.STAGE_WIDTH, 0.05 * Constants.STAGE_HEIGHT);
        }

        /**
         * Add the platforms to the scene
         */
        private function AddPlatforms():void
        {
            var platform:GameObject;

            for ( var i:Number = 0; i < Constants.PLATFORMS.length; ++i )
            {
                var r:Rectangle = Constants.PLATFORMS[i];

                platform = Layers.PlatformLayer.Add(GameObject);

                // if we're reusing one, we can probably just resize it an
                // not worry about drawing the rect
                platform.graphics.beginFill(Constants.PLATFORM_COLOR);
                platform.graphics.drawRect( 0, 0, r.width, r.height);
                platform.graphics.endFill();

                platform.x = r.x;
                platform.y = r.y;
            }
        }

        /**
         * Update and check timer
         */
        public override function Update():void
        {
            super.Update();

            // characer interaction
            Layers.ForEachCartesianProductStyle(Layers.PlayerLayer, Layers.EnemyLayer, PlayerEnemyCheck)
            Layers.ForEachCartesianProductStyle(Layers.PlayerLayer, Layers.NPCLayer, PlayerNPCCheck)

            Physics.Update();
        }

        /**
         * clean up
         */
        public override function End():void
        {
            super.End();

            // remove all those event listeners we set
            Layers.PlayerLayer.ForEach(RemoveDeathEventListener);
            Layers.NPCLayer.ForEach(RemoveDeathEventListener);

            // notably NOT doing this here
            //mGame.SetCameraFollowTarget(null);
            //mGame.ResetScrolling();
            //Layers.RemoveAll();

            // clen up any textfields we created
            for each (var textField:TextField in mTextFields)
            {
                mGame.GetDisplayObjectContainer().removeChild(textField);
            }
            mTextFields = null;
        }

        /**
         * clean up so that stuff gets garbage collected
         * and we don't have multiple listeners
         */
        private function RemoveDeathEventListener(character:PlatformerCharacter):void
        {
            character.removeEventListener(PlatformerCharacter.EVENT_DEATH, CharacterDied);
        }

        /**
         * react when a character dies
         */
        private function CharacterDied(e:Event = null):void
        {
            var character:PlatformerCharacter = e.target as PlatformerCharacter;
            character.dy = -Constants.DEATH_HOP_SPEED;
            character.dr = Constants.DEATH_ROTATION;
            LaunchBlood(character.x, character.y);

            character.removeEventListener(PlatformerCharacter.EVENT_DEATH, CharacterDied);
        }

        /**
         *
         */
        private function LaunchBlood(x:Number, y:Number):void
        {
            for (var i:uint = 0; i < Constants.BLOOD_PARTICLES_PER_DEATH; ++i)
            {
                var bloodParticle:GameObject = Layers.BloodLayer.Add(Blood);
                bloodParticle.x = x;
                bloodParticle.y = y;
            }
        }

        /**
         * check player-enemy collisions
         */
        private function PlayerEnemyCheck(player:Player, enemy:Enemy):void
        {
            if (!player.IsDying())
            {
                if (!enemy.IsDying())
                {
                    var ci:CollisionInfo = Util.CheckAabbCollision(player, enemy, Time.TimeDelta);

                    if (null != ci)
                    {
                        var collidedWithEnemyInFront:Boolean = Util.OppositeSign(ci.normal.x, player.scaleX);
                        var punchedHim:Boolean = player.IsPunching && collidedWithEnemyInFront;

                        enemy.StartDeath();

                        if (punchedHim)
                        {
                            ++mEnemiesPunched;
                            UpdateScore();
                            enemy.dx = player.scaleX * Constants.PUNCHED_ENEMY_VELOCITY.x;
                            enemy.dy = Constants.PUNCHED_ENEMY_VELOCITY.y;
                        }
                        else
                        {
                            LaunchBlood(player.x, player.y);
                        }
                    }
                }
            }
        }

        /**
         * check player-npc collisions
         */
        private function PlayerNPCCheck(player:Player, npc:Family):void
        {
            if (!player.IsDying())
            {
                if (!npc.IsDying())
                {
                    var ci:CollisionInfo = Util.CheckAabbCollision(player, npc, Time.TimeDelta);

                    if (null != ci)
                    {
                        var collidedWithNPCInFront:Boolean = Util.OppositeSign(ci.normal.x, player.scaleX);
                        var punchedHim:Boolean = player.IsPunching && collidedWithNPCInFront;

                        if (punchedHim)
                        {
                            npc.StartDeath();
                            mGame.SwitchToScript(new Lose());
                        }
                    }
                }
            }
        }

        /**
         *
         */
        protected override function DoBeatAction(beat:Number):void
        {
            super.DoBeatAction(beat);

            if ( (beat % 4) == 0)
            {
                var character:MiniLD10Character;

                if (Math.random() < Constants.CHANCE_FOR_FAMILY_SPAWN)
                {
                    character = Layers.NPCLayer.Add(Family) as MiniLD10Character;
                }
                else
                {
                    character = Layers.EnemyLayer.Add(Enemy) as MiniLD10Character;
                }

                character.addEventListener(PlatformerCharacter.EVENT_DEATH, CharacterDied);

                character.x = Util.Lerp(Constants.SPAWN_AREA.left, Constants.SPAWN_AREA.right, Math.random())
                character.y = Util.Lerp(Constants.SPAWN_AREA.top, Constants.SPAWN_AREA.bottom, Math.random())
            }
        }
    }
}
