﻿package meyermike.script
{
    import flash.events.Event;
    import flash.events.EventDispatcher;
    import meyermike.GameInterface;
    import meyermike.script.Step;
    import meyermike.script.step.Loop;
    import meyermike.script.step.Parallel;
    import meyermike.script.step.RunFunc;
    import meyermike.script.step.Wait;
    import meyermike.Time;

    /**
     * ...
     * @author Mike Meyer
     */
    public class Script extends EventDispatcher
    {
        public static var EVENT_BEAT:String    = "beat";
        public static var EVENT_OFFBEAT:String = "offbeat";
        public static var EVENT_QUARTERBEAT:String = "quarterbeat";

        public var mBeatsPerMinute:Number = 60.0;

        private var mTimeSinceLastQuarterBeat:Number;
        private var mCurQuarterBeat:Number;

        protected var mGame:GameInterface;
        private var mScript:Step;

        public function Script()
        {
            mTimeSinceLastQuarterBeat = 0;
            mCurQuarterBeat = 0;
        }

        /**
         * "heartbeat" function. called every frame
         */
        public function Update():void
        {
            mTimeSinceLastQuarterBeat += Time.TimeDelta;

            if (mTimeSinceLastQuarterBeat*4 >= GetSecondsPerBeat())
            {
                QuarterBeat();
            }
        }

        /**
         * called every 1/4 beat (from Update)
         */
        private function QuarterBeat():void
        {
            mTimeSinceLastQuarterBeat = 0;
            ++mCurQuarterBeat;

            // send event
            if (1 == (mCurQuarterBeat % 4))
            {
                // execute next step
                DoBeatAction(GetCurBeat())

                dispatchEvent(new Event(EVENT_BEAT));
            }
            else if (3 == (mCurQuarterBeat % 4))
            {
                dispatchEvent(new Event(EVENT_OFFBEAT));
            }

            dispatchEvent(new Event(EVENT_QUARTERBEAT));
        }

        /**
         * override me!
         * Do whatever is supposed to happen at this beat
         */
        protected function DoBeatAction(curBeat:Number):void
        {
            mScript.Execute();
        }

        /**
         * Get the beat we are currently on
         */
        public function GetCurBeat():Number
        {
            return Math.ceil(mCurQuarterBeat / 4);
        }

        /**
         * Get the part of the beat we are currently on
         */
        public function GetCurQuarterBeat():Number
        {
            return mCurQuarterBeat;
        }

        /**
         * Set the current beat
         */
        public function SetCurBeat(beat:Number):void
        {
            mCurQuarterBeat = 1 + (beat * 4);
        }

        /**
         * Go back to the beginning
         */
        public function Restart():void
        {
            mCurQuarterBeat = 0;
            End();
            Begin(mGame);
            mScript.Restart();
        }

        /**
         * get the script ready to be played
         */
        public function Begin(game:GameInterface):void
        {
            mGame = game;

            if (null == mScript)
            {
                mScript = CreateScript();
            }
            else
            {
                mScript.Restart();
            }
        }

        /**
         * clean up after the script
         */
        public function End():void
        {
            if (null == mScript)
            {
                mScript = CreateScript();
            }
            else
            {
                mScript.Restart();
            }
        }

        /**
         * access to the current BPM
         */
        public function GetBPM():Number
        {
            return mBeatsPerMinute;
        }

        /**
         * access to how long each beat is
         */
        public function GetSecondsPerBeat():Number
        {
            return 60.0 / mBeatsPerMinute;
        }

        /**
         * override me!
         */
        protected function CreateScript():Step
        {
            var steps:/*Step*/Array = [];
            return new Loop(1, steps);
        }
    }
}
