﻿package meyermike.widget
{
    import flash.display.Sprite;
    import flash.display.DisplayObjectContainer;
    import flash.geom.Rectangle;
    import meyermike.constants.Colors;
    import meyermike.GameObject;
    import meyermike.Util;

    /**
     * ...
     * @author Mike Meyer
     */
    public class PulsingText extends GameObject
    {
        protected var mPulseHolder:DisplayObjectContainer;
        protected var mPulseTextField:TextField;
        protected var mTextField:TextField;

        protected var mColor:Number = Colors.BLACK;

        private static const PULSE_MAX_SCALE:Number = 1.25;
        private static const PULSE_MAX_ALPHA:Number = 0.5;
        private static const PULSE_MIN_ALPHA:Number = 0.0;
        private static const PULSE_DURATION:Number = 0.75;

        private var mPulseTimer:Number;

        public function PulsingText()
        {
            super();

            ResetDefaults();
        }

        public override function Init():void
        {
            super.Init();

            CreateGraphics();
            SetFontColor(mColor);
        }

        public function SetFontColor(color:Number):void
        {
            mColor = color;
            mPulseTextField.SetFontColor(mColor);
            mTextField.SetFontColor(mColor);
        }

        protected function CreateGraphics():void
        {
            if (null == mTextField)
            {
                mTextField = new TextField();
                this.addChild(mTextField);
            }

            if (null == mPulseHolder)
            {
                mPulseHolder = new Sprite();
                this.addChildAt(mPulseHolder, 0);
            }

            if (null == mPulseTextField)
            {
                mPulseTextField = new TextField();
                mPulseTextField.CenterOn(0, 0);
                mPulseHolder.addChild(mPulseTextField);
            }

            ResetPulseHolder();
        }

        /**
         * update the pulse holder's position
         */
        private function ResetPulseHolder():void
        {
            var bounds:Rectangle = mTextField.getBounds(this);
            mPulseHolder.x = bounds.width * 0.5;
            mPulseHolder.y = bounds.height * 0.5;
        }

        public override function Beat(beat:Number):void
        {
            super.Beat(beat);

            mPulseTimer = 0;
        }

        public override function Update(elapsedTime:Number):void
        {
            super.Update(elapsedTime);

            mPulseTimer += elapsedTime;

            var t:Number = Math.min(1.0, mPulseTimer / PULSE_DURATION);
            mPulseHolder.alpha = Util.Lerp(PULSE_MAX_ALPHA, PULSE_MIN_ALPHA, t);
            mPulseHolder.scaleX = Util.Lerp( 1.0, PULSE_MAX_SCALE, t);
            mPulseHolder.scaleY = mPulseHolder.scaleX;
        }

        /**
         * center the textfield on some point.  CALL SET TEXT *FIRST*
         */
        public function CenterOn(x:Number, y:Number):void
        {
            var bounds:Rectangle = this.getBounds(this.parent);
            this.x = x - (bounds.width * 0.5);
            this.y = y - (bounds.height * 0.5);
        }

        /**
         * Change the displayed text
         */
        public function SetText(text:String):void
        {
            mPulseTextField.SetText(text);
            mPulseTextField.CenterOn(0,0);

            mTextField.SetText(text);
            ResetPulseHolder();
        }

        /**
         * Change the displayed font size.
         */
        public function SetFontSize(desiredSize:Number):void
        {
            mPulseTextField.SetFontSize(desiredSize);
            mPulseTextField.CenterOn(0,0);

            mTextField.SetFontSize(desiredSize);
            ResetPulseHolder();
        }

        /**
         * reset to defaults.  This thing will likely get re-used!
         */
        public override function Shutdown():void
        {
            ResetDefaults();
            super.Shutdown();
        }

        /**
         * reset to defaults.  This thing will likely get re-used!
         */
        private function ResetDefaults():void
        {
            mPulseTimer = PULSE_DURATION;

            if (null != mPulseTextField)
            {
                mPulseTextField.ResetDefaults();
            }

            if (null != mTextField)
            {
                mTextField.ResetDefaults();
            }
        }

        /**
         * Change the font.  WILL SCREW UP ALIGNMENT.
         */
        public function SetFont(font:String):void
        {
            mTextField.SetFont(font);
            mPulseTextField.SetFont(font);

            ResetPulseHolder();
        }
    }

}
